@@ -5,6 +5,7 @@ from matplotlib import pyplot as plt
 from scipy.signal import savgol_filter, find_peaks
 from scipy.interpolate import interp1d
 from scipy.optimize import curve_fit
+from scipy.stats import linregress
 from collections import namedtuple
 import time
 import pandas as pd
@@ -12,6 +13,7 @@ import pandas as pd
 from sekve.plotting import update_rcParams
 from sekve.model import sEKVModel
 from sekve.extractor.ss_extractor import extract_ss, extract_ss_cryo
+from sekve.extractor.gds_extractor import extract_Gds
 from sekve.utils import get_savgol_win_num
 from sekve.plotting import update_rcParams
 
@@ -21,6 +23,7 @@ log = logging.getLogger(__name__)
 class Base(sEKVModel):
     _progress_figsize = (7, 5)
     _final_figsize = (4, 3)
+    _output_figsize = (6, 3)
 
     def __init__(self, vg: ndarray, i: ndarray, vd: float,
                  width: float = None, length: float = None,
@@ -35,9 +38,11 @@ class Base(sEKVModel):
         self._Gm = self.cal_Gm(i=self.ID, vg=self.VG)
         self._Gm_over_ID = self.cal_Gm_over_ID(i=self.ID, vg=self.VG)
         self._n_values = self.cal_n(i=self.ID, vg=self.VG, temp=self.T)
+        self._gDS = None
 
         self._progress_fig = plt.figure(figsize=self.__class__._progress_figsize)
         self._final_fig = plt.figure(figsize=self.__class__._final_figsize)
+        self._output_fig = plt.figure(figsize=self.__class__._output_figsize)
 
     @property
     def W(self):
@@ -85,6 +90,11 @@ class Base(sEKVModel):
         self._final_fig.tight_layout()
         return self._final_fig
 
+    @property
+    def output_fig(self):
+        self._output_fig.tight_layout()
+        return self._output_fig
+
     @classmethod
     def set_progress_figsize(cls, figsize: tuple):
         cls._progress_figsize = figsize
@@ -176,7 +186,52 @@ class Extractor(Params):
                  force_lambdac_0=False, opt_method="trf", no_refine=False,
                  transconductance_cleanning=False
                  ):
-        # TODO: Document the parameters.
+        """ An automated extractor for sEKV MOSFET model.
+
+        Parameters
+        ----------
+        vg: numpy.ndarray
+            A sequence of gate voltage. Please note that it is the gate-to-source voltage for nMOS and
+            source-to-gate voltage for pMOS.
+        i: numpy.ndarray
+            A sequence of drain current. Please note that it is the drain-to-source current for nMOS and
+            source-to-drain current for pMOS.
+        vd: float
+            Drain-to-bulk voltage for nMOS and source-to-bulk voltage for pMOS.
+        vs: float
+            Source-to-bulk voltage for nMOS and bulk-to-source voltage for pMOS.
+        width: float, optional
+            Device width in meter.
+        length: float, optional
+            Device length in meter.
+        temp: float, optional
+            Ambient temperature in Kelvin.
+        remove_mobility_reduction: bool, optional
+            If True, the data point suffering the mobility reduction is filter out.
+        vth_tol: float, optional
+            A tolerance in Volt when extractor re-tunes the `Ispec` from the subthreshold regime. If
+            data points in subthreshold regime are sufficient, clean, and reasonably linear, you could set `vth_tol=0`.
+        n_ext_method: {"ss", "ss_general"}, optional
+            Method extracting subthreshold swing:
+
+                * 'ss' (default): the old method validated only at room temperature.
+                * 'ss_general': the new method validated at various temperature.
+                This method requires `width` and `length`.
+
+        force_lambdac_0: bool, optional
+            If True, force `lambdac=0`.
+        opt_method: {'trf', 'dogbox}, optional
+            See `scipy.optimize.least_squares` for more information.
+        no_refine: bool, optional
+            If True, extractor won't re-tune Ispec from subthreshold regime.
+        transconductance_cleanning: bool, optional
+            If True, the data region having negative transconductance would be filter out.
+
+        Raises
+        ------
+        ValueError
+        IndexError
+        """
         vg, i = self._user_cleanup(vg, i, transconductance_cleanning)
         super().__init__(vg, i, vd, width, length, temp, vs)
 
@@ -187,11 +242,17 @@ class Extractor(Params):
         self._opt_method = opt_method
         self._no_refine = no_refine
 
-    def _user_cleanup(self, vg: ndarray, i: ndarray, transconductance_cleanning):
+        # check
+        if self._n_ext_method == "ss_general":
+            if self.W is None or self.L is None:
+                raise ValueError('Device `width` and `length` should be given, if `n_ext_method="ss_general"`.')
+
+    @staticmethod
+    def _user_cleanup(vg: ndarray, i: ndarray, transconductance_cleanning: bool):
         """Cleaning the input values from the user."""
         loc = np.zeros(i.size, dtype=bool)
-        for j,i in enumerate(i[::-1]):
-            if i < 0 :
+        for j, _i in enumerate(i[::-1]):
+            if _i < 0:
                 break
             loc[loc.size-j-1] = True
 
@@ -230,6 +291,100 @@ class Extractor(Params):
         spent_time = end - start
         log.info(f"Extraction is finished, spent {spent_time} s")
 
+    def extract_sigmad_lambdad(self, df: pd.DataFrame,
+                               gds_ext_dict: dict = None):
+        ax1 = self._output_fig.add_subplot(121)
+        ax2 = self._output_fig.add_subplot(122)
+
+        if self.VDS is None:
+            raise ValueError("Drain-to-source voltage 'vds' is not given from class.")
+        gds_ext = {
+            "method": "deriv",
+            "at_vd": self.VDS
+        }
+        if gds_ext_dict is not None:
+            for k, v in gds_ext_dict.items():
+                gds_ext[k] = v
+
+        gdss = []
+        ics = []
+        gms_model = []
+        gms_data = []
+
+        for vg, idvd in df.items():
+            Gds = extract_Gds(idvd.values, idvd.index.values, **gds_ext)
+            gds = Gds / self.Gspec
+
+            x = self.VG
+            y = self.ID
+            f = interp1d(x, y, fill_value="extrapolate")
+            id = f(float(vg))
+            ic = id / self.Ispec
+
+            y = self.Gm
+            f = interp1d(x, y, fill_value="extrapolate")
+            gm = f(float(vg))
+            gms_data.append(gm * self.n / self.Gspec)
+
+            gdss.append(gds)
+            ics.append(ic)
+
+            # model
+            gms_ = self.model_gms(id, self.lambdac, self.Ispec)
+            gms_model.append(gms_)
+
+        gdss = np.array(gdss)
+        self._gDS = gdss
+        ics = np.array(ics)
+        gms_model = np.array(gms_model)
+        gms_data = np.array(gms_data)
+
+        mk = np.where(ics <= 0.05)
+        slope, _, _, _, _ = linregress(gms_data[mk], gdss[mk])
+        sigma_d = slope * self._n
+        exp_line = ics * sigma_d / self._n
+        ax1.loglog(ics, gdss, marker="o", ls="", mfc="none", color="k", label="data")
+        ax1.loglog(ics, exp_line, "r-", label="Ext. of $\sigma_d$")
+        ax1.grid(axis="both", which="major", ls="-", alpha=0.5)
+        ax1.grid(axis="both", which="minor", ls="--", alpha=0.5)
+        ax1.set_xlabel("$IC$")
+        ax1.set_ylabel("$g_{DS}$")
+        ax1.legend(loc=0)
+
+        f = lambda _id, _lambda: self.model_gms(i=_id, lambdac=_lambda, ispec=self.Ispec) * slope
+        res = curve_fit(f,
+                        xdata=ics * self.Ispec,
+                        ydata=gdss,
+                        p0=[0.6],
+                        loss="linear",
+                        bounds=([0], [1]),
+                        method='trf')
+        lambdad = res[0][0]
+        gds_use_lambdac = slope * gms_model
+        gds_use_lambdad = slope * self.model_gms(
+            i=ics * self.Ispec, lambdac=lambdad, ispec=self.Ispec)
+
+        factor = self._n / (sigma_d * ics)
+        ax2.loglog(ics, gdss * factor, marker="o", ls="", color="k", label="data", mfc="none")
+        ax2.loglog(ics, gds_use_lambdac * factor, ls="-", color="r", label="$\lambda_d$ = $\lambda_c$")
+        ax2.loglog(ics, gds_use_lambdad * factor, ls="-", color="b", label="optimzed $\lambda_d$")
+        ax2.grid(axis="both", which="major", ls="-", alpha=0.5)
+        ax2.grid(axis="both", which="minor", ls="--", alpha=0.5)
+        s = [
+            "n = %.2f" % self._n,
+            "$\sigma_d$ = %.3f" % sigma_d,
+            "$\lambda_c$ = %.3f" % self._lambdac,
+            "$\lambda_d$ = %.3f" % lambdad,
+        ]
+        ax2.text(x=0.03, y=0.03, s="\n".join(s), va="bottom", ha="left",
+                 transform=ax2.transAxes)
+        ax2.set_xlabel("IC")
+        ax2.set_ylabel("$ng_{DS}$/$\sigma_dIC$")
+        ax2.legend(loc=4)
+
+        self._lambdad = lambdad
+        self._sigmad = sigma_d
+
     def _get_Gm_peak(self, show=False):
         """Get peak Gm."""
         log_gm = np.log(self.Gm)
@@ -319,7 +474,7 @@ class Extractor(Params):
         return loc
 
     def _extract_n(self):
-        if self._n_ext_method in ["ss","ss_cryo"]:
+        if self._n_ext_method in ["ss", "ss_general"]:
             if self._n_ext_method == "ss":
                 ss, vc = extract_ss(
                     v=self.VG, i=self.ID, t=self.T,
